package com.traffic.kafka.producer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.UUID;

import javax.annotation.PostConstruct;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.KafkaHeaders;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.stereotype.Service;

import com.traffic.kafka.vo.Vehicle;

@Service
public class TrafficDataSender {
	private static final Logger LOG = LoggerFactory.getLogger(TrafficDataSender.class);

    @Autowired
    private KafkaTemplate<String, Vehicle> kafkaTemplate;

    @Value("${com.kafka.topic}")
    private String topic;

    public void send(Vehicle data){
        LOG.info("sending data='{}' to topic='{}'", data, topic);

        Message<Vehicle> message = MessageBuilder
                .withPayload(data)
                .setHeader(KafkaHeaders.TOPIC, topic)
                .build();
        
        kafkaTemplate.send(message);
    }
    @PostConstruct
    public void generateIoTEvent() throws InterruptedException {
		List<String> routeList = Arrays.asList(new String[]{"Route-37", "Route-43", "Route-82"});
		List<String> vehicleTypeList = Arrays.asList(new String[]{"Large Truck", "Small Truck", "Private Car", "Bus", "Taxi"});
		Random rand = new Random();
		//logger.info("Sending events");
		// generate event in loop
		while (true) {
			List<Vehicle> eventList = new ArrayList<Vehicle>();
			for (int i = 0; i < 100; i++) {// create 100 vehicles
				String vehicleId = UUID.randomUUID().toString();
				String vehicleType = vehicleTypeList.get(rand.nextInt(5));
				String routeId = routeList.get(rand.nextInt(3));
				Date timestamp = new Date();
				double speed = rand.nextInt(100 - 20) + 20;// random speed between 20 to 100
				double fuelLevel = rand.nextInt(40 - 10) + 10;
				for (int j = 0; j < 5; j++) {// Add 5 events for each vehicle
					String coords = getCoordinates(routeId);
					String latitude = coords.substring(0, coords.indexOf(","));
					String longitude = coords.substring(coords.indexOf(",") + 1, coords.length());
					Vehicle event = new Vehicle(vehicleId, vehicleType, routeId, latitude, longitude, timestamp, speed,fuelLevel);
					eventList.add(event);
				}
			}
			Collections.shuffle(eventList);// shuffle for random events
			for (Vehicle event : eventList) {
				send(event);
				Thread.sleep(rand.nextInt(3000 - 1000) + 1000);//random delay of 1 to 3 seconds
			}
		}
	}
	
	//Method to generate random latitude and longitude for routes
	private String  getCoordinates(String routeId) {
		Random rand = new Random();
		int latPrefix = 0;
		int longPrefix = -0;
		if (routeId.equals("Route-37")) {
			latPrefix = 33;
			longPrefix = -96;
		} else if (routeId.equals("Route-82")) {
			latPrefix = 34;
			longPrefix = -97;
		} else if (routeId.equals("Route-43")) {
			latPrefix = 35;
			longPrefix = -98;
		} 
		Float lati = latPrefix + rand.nextFloat();
		Float longi = longPrefix + rand.nextFloat();
		return lati + "," + longi;
	}
}
